@@ -6,7 +6,6 @@
 // KIND, either express or implied. Please review the Licences for the specific language governing
 // permissions and limitations relating to use of the SAFE Network Software.
 
-mod elder_candidates;
 mod errors;
 mod node_info;
 pub mod node_state;
@@ -14,13 +13,13 @@ pub mod prefix_map;
 pub mod section_authority_provider;
 pub mod section_keys;
 mod section_peers;
+pub mod utils;
 
 #[cfg(feature = "test-utils")]
 pub use self::section_authority_provider::test_utils;
 
 pub use self::section_keys::{SectionKeyShare, SectionKeysProvider};
 
-pub use elder_candidates::ElderCandidates;
 pub use node_info::NodeInfo;
 pub use node_state::NodeState;
 pub use section_authority_provider::{SectionAuthUtils, SectionAuthorityProvider};
@@ -28,14 +27,14 @@ pub use section_authority_provider::{SectionAuthUtils, SectionAuthorityProvider}
 use crate::messaging::system::{KeyedSig, SectionAuth, SectionPeers as SectionPeersMsg};
 use prefix_map::NetworkPrefixMap;
 // use crate::node::{dkg::SectionAuthUtils, recommended_section_size};
-use crate::types::{log_markers::LogMarker, Peer};
+use crate::types::Peer;
 pub use errors::{Error, Result};
 
 use bls::PublicKey as BlsPublicKey;
 use section_peers::SectionPeers;
 use secured_linked_list::SecuredLinkedList;
 use serde::Serialize;
-use std::{collections::BTreeSet, convert::TryInto, iter, net::SocketAddr, sync::Arc};
+use std::{collections::BTreeSet, iter, net::SocketAddr, sync::Arc};
 use tokio::sync::RwLock;
 use xor_name::{Prefix, XorName};
 
@@ -214,7 +213,8 @@ impl NetworkKnowledge {
         genesis_sk_set: bls::SecretKeySet,
     ) -> Result<(NetworkKnowledge, SectionKeyShare)> {
         let public_key_set = genesis_sk_set.public_keys();
-        let secret_key_share = genesis_sk_set.secret_key_share(0);
+        let secret_key_index = 0u8;
+        let secret_key_share = genesis_sk_set.secret_key_share(secret_key_index as u64);
         let genesis_key = public_key_set.public_key();
 
         let section_auth =
@@ -581,56 +581,6 @@ impl NetworkKnowledge {
         self.signed_sap.read().await.clone()
     }
 
-    /// Generate a new section info(s) based on the current set of members,
-    /// excluding any member matching a name in the provided `excluded_names` set.
-    /// Returns a set of candidate SectionAuthorityProviders.
-    pub async fn promote_and_demote_elders(
-        &self,
-        our_name: &XorName,
-        excluded_names: &BTreeSet<XorName>,
-    ) -> Vec<ElderCandidates> {
-        if let Some((our_elder_candidates, other_elder_candidates)) =
-            self.try_split(our_name, excluded_names).await
-        {
-            return vec![our_elder_candidates, other_elder_candidates];
-        }
-
-        // Candidates for elders out of all the nodes in the section, even out of the
-        // relocating nodes if there would not be enough instead.
-        let sap = self.authority_provider().await;
-        let expected_peers =
-            self.section_peers
-                .elder_candidates(elder_count(), &sap, excluded_names, None);
-        info!(
-            ">>>> ELDER CANDIDATES {}: {:?}",
-            expected_peers.len(),
-            expected_peers
-        );
-        let expected_names: BTreeSet<_> = expected_peers.iter().map(Peer::name).collect();
-        let current_names: BTreeSet<_> = sap.names();
-
-        if expected_names == current_names {
-            vec![]
-        } else if expected_names.len()
-            < crate::network_knowledge::supermajority(current_names.len())
-        {
-            warn!("ignore attempt to reduce the number of elders too much");
-            vec![]
-        } else if expected_names.len() < current_names.len() {
-            // Could be due to the newly promoted elder doesn't have enough knowledge of
-            // existing members.
-            warn!("Ignore attempt to shrink the elders");
-            trace!("current_names  {:?}", current_names);
-            trace!("expected_names {:?}", expected_names);
-            trace!("excluded_names {:?}", excluded_names);
-            trace!("section_peers {:?}", self.section_peers);
-            vec![]
-        } else {
-            let elder_candidates = ElderCandidates::new(sap.prefix(), expected_peers);
-            vec![elder_candidates]
-        }
-    }
-
     /// Prefix of our section.
     pub async fn prefix(&self) -> Prefix {
         self.signed_sap.read().await.prefix()
@@ -696,11 +646,6 @@ impl NetworkKnowledge {
         self.section_peers.is_member(name)
     }
 
-    /// Returns whether the given peer is already relocated to our section.
-    pub async fn is_relocated_to_our_section(&self, name: &XorName) -> bool {
-        self.section_peers.is_relocated_to_our_section(name)
-    }
-
     pub async fn find_member_by_addr(&self, addr: &SocketAddr) -> Option<Peer> {
         self.section_peers
             .members()
@@ -708,91 +653,6 @@ impl NetworkKnowledge {
             .find(|info| info.addr() == *addr)
             .map(|info| *info.peer())
     }
-
-    // Tries to split our section.
-    // If we have enough nodes for both subsections, returns the SectionAuthorityProviders
-    // of the two subsections. Otherwise returns `None`.
-    async fn try_split(
-        &self,
-        our_name: &XorName,
-        excluded_names: &BTreeSet<XorName>,
-    ) -> Option<(ElderCandidates, ElderCandidates)> {
-        trace!("{}", LogMarker::SplitAttempt);
-        if self.authority_provider().await.elder_count() < elder_count() {
-            trace!("No attempt to split as our section does not have enough elders.");
-            return None;
-        }
-
-        let (prefix_next_bit, our_new_size, sibling_new_size) =
-            self.get_split_info(our_name, excluded_names).await?;
-
-        debug!(
-            "Upon section split attempt: our section size {:?}, theirs {:?}",
-            our_new_size, sibling_new_size
-        );
-
-        let sap = self.authority_provider().await;
-
-        let our_prefix = self.prefix().await.pushed(prefix_next_bit);
-        let our_elders = self.section_peers.elder_candidates(
-            elder_count(),
-            &sap,
-            excluded_names,
-            Some(&our_prefix),
-        );
-
-        let other_prefix = self.prefix().await.pushed(!prefix_next_bit);
-        let other_elders = self.section_peers.elder_candidates(
-            elder_count(),
-            &sap,
-            excluded_names,
-            Some(&other_prefix),
-        );
-
-        let our_elder_candidates = ElderCandidates::new(our_prefix, our_elders);
-        let other_elder_candidates = ElderCandidates::new(other_prefix, other_elders);
-
-        Some((our_elder_candidates, other_elder_candidates))
-    }
-
-    pub async fn get_split_info(
-        &self,
-        our_name: &XorName,
-        excluded_names: &BTreeSet<XorName>,
-    ) -> Option<(bool, usize, usize)> {
-        let (next_bit_index, prefix_next_bit) =
-            if let Ok(index) = self.prefix().await.bit_count().try_into() {
-                let prefix_next_bit = our_name.bit(index);
-                (index, prefix_next_bit)
-            } else {
-                // Already at the longest prefix, can't split further.
-                warn!("We cannot split as we are at longest prefix possible");
-                return None;
-            };
-
-        let (our_new_size, sibling_new_size) = self
-            .section_peers
-            .members()
-            .iter()
-            .filter(|info| !excluded_names.contains(&info.name()))
-            .map(|info| info.name().bit(next_bit_index) == prefix_next_bit)
-            .fold((0, 0), |(ours, siblings), is_our_prefix| {
-                if is_our_prefix {
-                    (ours + 1, siblings)
-                } else {
-                    (ours, siblings + 1)
-                }
-            });
-
-        // If none of the two new sections would contain enough entries, return `None`.
-        if our_new_size < recommended_section_size()
-            || sibling_new_size < recommended_section_size()
-        {
-            return None;
-        }
-
-        Some((prefix_next_bit, our_new_size, sibling_new_size))
-    }
 }
 
 // Create `SectionAuthorityProvider` for the first node.
@@ -801,8 +661,12 @@ fn create_first_section_authority_provider(
     sk_share: &bls::SecretKeyShare,
     peer: Peer,
 ) -> Result<SectionAuth<SectionAuthorityProvider>> {
-    let section_auth =
-        SectionAuthorityProvider::new(iter::once(peer), Prefix::default(), pk_set.clone());
+    let section_auth = SectionAuthorityProvider::new(
+        iter::once(peer),
+        Prefix::default(),
+        [NodeState::joined(peer, None)],
+        pk_set.clone(),
+    );
     let sig = create_first_sig(pk_set, sk_share, &section_auth)?;
     Ok(SectionAuth::new(section_auth, sig))
 }
